"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "PortalView", {
  enumerable: true,
  get: function () {
    return _PortalView.default;
  }
});
exports.unsubscribe = exports.subscribe = exports.register = exports.publish = exports.addPortal = void 0;

var _reactNative = require("react-native");

var _PortalView = _interopRequireDefault(require("./PortalView"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const {
  IONPortalsPubSub,
  IONPortalManager
} = _reactNative.NativeModules;
const PortalsPubSub = new _reactNative.NativeEventEmitter(IONPortalsPubSub);
const subscriptionMap = new Map();

const subscribe = async (topic, onMessageReceived) => {
  const subscriptionRef = await IONPortalsPubSub.subscribe(topic);
  const subscriber = PortalsPubSub.addListener('PortalsSubscription', message => {
    if (message.subscriptionRef === subscriptionRef) {
      onMessageReceived(message);
    }
  });
  subscriptionMap.set(subscriptionRef, subscriber);
  return subscriptionRef;
};

exports.subscribe = subscribe;

const unsubscribe = (topic, subRef) => {
  IONPortalsPubSub.unsubscribe(topic, subRef);
  const subscription = subscriptionMap.get(subRef);

  if (subscription !== undefined) {
    subscription.remove();
    subscriptionMap.delete(subRef);
  }
};

exports.unsubscribe = unsubscribe;

const publish = (topic, data) => {
  const msg = {
    message: data
  };
  IONPortalsPubSub.publish(topic, msg);
};

exports.publish = publish;

const register = key => {
  IONPortalManager.register(key);
};

exports.register = register;

const addPortal = portal => {
  IONPortalManager.addPortal(portal);
};

exports.addPortal = addPortal;
//# sourceMappingURL=index.js.map