import { NativeEventEmitter, NativeModules } from 'react-native';
const {
  IONPortalsPubSub,
  IONPortalManager
} = NativeModules;
export { default as PortalView } from './PortalView';
const PortalsPubSub = new NativeEventEmitter(IONPortalsPubSub);
const subscriptionMap = new Map();
export const subscribe = async (topic, onMessageReceived) => {
  const subscriptionRef = await IONPortalsPubSub.subscribe(topic);
  const subscriber = PortalsPubSub.addListener('PortalsSubscription', message => {
    if (message.subscriptionRef === subscriptionRef) {
      onMessageReceived(message);
    }
  });
  subscriptionMap.set(subscriptionRef, subscriber);
  return subscriptionRef;
};
export const unsubscribe = (topic, subRef) => {
  IONPortalsPubSub.unsubscribe(topic, subRef);
  const subscription = subscriptionMap.get(subRef);

  if (subscription !== undefined) {
    subscription.remove();
    subscriptionMap.delete(subRef);
  }
};
export const publish = (topic, data) => {
  const msg = {
    message: data
  };
  IONPortalsPubSub.publish(topic, msg);
};
export const register = key => {
  IONPortalManager.register(key);
};
export const addPortal = portal => {
  IONPortalManager.addPortal(portal);
};
//# sourceMappingURL=index.js.map